# -*- coding: utf-8 -*-

from django.http.response import Http404
from django.conf import settings
from django.db.models.query import Prefetch
from django.views.generic import TemplateView
from django.shortcuts import render
from django_ajax.mixin import AJAXMixin
from alapage.models import Page
from alapage.utils import can_see_page
from alapage.conf import BASE_TEMPLATE_PATH


class AddPagePostView(AJAXMixin, TemplateView):
    template_name = "alapage/wizard/tree_inline.html"
    
    def dispatch(self, request, *args, **kwargs):
        if not request.user.has_perm('alapage.change_page') or not self.request.method == "POST":
            raise Http404
        title = self.request.POST['title']
        url = self.request.POST['url']
        parent_url = self.request.POST['parent']
        parent = Page.objects.get(url=parent_url)
        self.newnode = Page.objects.create(url=url, title=title, parent=parent, editor=request.user)
        self.context=self.get_context_data()
        return super(AddPagePostView, self).dispatch(request, *args, **kwargs)
    
    def get_context_data(self, **kwargs):
        context = super(AddPagePostView, self).get_context_data(**kwargs)
        context["template_to_extend"] = BASE_TEMPLATE_PATH
        root_node = Page.objects.get(url="/")
        nodes = root_node.get_descendants(include_self=True)
        context['nodes'] = nodes
        context["flashnode"] = self.newnode.pk
        return context
    
    def post(self, request, *args, **kwargs):
        data = render(request, "alapage/wizard/tree_inline.html", context=self.context)
        return data


class PageWizardView(TemplateView):
    template_name = "alapage/wizard/index.html"
    
    def dispatch(self, request, *args, **kwargs):
        if not request.user.has_perm('alapage.change_page'):
            raise Http404
        return super(PageWizardView, self).dispatch(request, *args, **kwargs)
    
    def get_context_data(self, **kwargs):
        context = super(PageWizardView, self).get_context_data(**kwargs)
        context["template_to_extend"] = BASE_TEMPLATE_PATH
        root_node, created = Page.objects.get_or_create(url="/")
        nodes = root_node.get_descendants(include_self=True)
        context['nodes'] = nodes
        return context


class PageView(TemplateView):
    
    def dispatch(self, request, *args, **kwargs):
        # get the page
        try:
            url = kwargs['url']
        except:
            url = '/'
        if not url.startswith('/'):
            url = '/' + url
        if not url.endswith('/') and settings.APPEND_SLASH:
            url += '/'
        # get the page with 1 query
        self.page_q = Page.objects.filter(url=url)
        if len(self.page_q) == 0:
            raise Http404
        self.page = self.page_q[0]
        # check if other queries are really necessary and get the related data
        if self.page.is_reserved_to_users is True \
        or self.page.is_reserved_to_groups is True\
        or self.page.staff_only is True \
        or self.page.superuser_only is True:
            if self.page.is_reserved_to_users is True:
                prefetch = Prefetch('users_only', queryset=self.page_q)
                self.page_q = Page.objects.prefetch_related(prefetch)
            if self.page.is_reserved_to_groups is True:
                prefetch = Prefetch('groups_only', queryset=self.page_q)
                self.page_q = Page.objects.prefetch_related(prefetch)
            # check permissions
            if can_see_page(self.page, request.user) is False:
                raise Http404
        return super(PageView, self).dispatch(request, *args, **kwargs)
    
    def get_template_names(self):
        template_name = 'alapage/default.html'
        if self.page.template_name:
            template_name=self.page.template_name
        return [template_name]
    
    def get_context_data(self, **kwargs):
        context = super(PageView, self).get_context_data(**kwargs)
        page=self.page
        if not page.published and not self.request.user.is_superuser:
            raise Http404
        context['page'] = page
        context['template_to_extend'] = BASE_TEMPLATE_PATH
        return context


class PagesmapView(TemplateView):
    template_name = "alapage/map.html"
    
    def get_context_data(self, **kwargs):
        context = super(PagesmapView, self).get_context_data(**kwargs)
        context["template_to_extend"] = BASE_TEMPLATE_PATH
        root_node, created = Page.objects.get_or_create(url="/")
        if created is True:
            root_node.title = "Home"
            root_node.save()
        nodes = root_node.get_descendants(include_self=True).filter(is_reserved_to_groups=False, staff_only=False, superuser_only=False, registration_required=False, is_reserved_to_users=False, published=True)
        context['nodes'] = nodes
        return context


class HomepageView(PageView):
    pass
    


# -*- coding: utf-8 -*-

from django import forms
from ckeditor_uploader.widgets import CKEditorUploadingWidget
from alapage.models import Page
from alapage.conf import EDIT_MODE, CODEMIRROR_KEYMAP
if EDIT_MODE == "code":
    from codemirror2.widgets import CodeMirrorEditor


class PageAdminForm(forms.ModelForm):
    
    if EDIT_MODE == 'visual':    
        content = forms.CharField(widget=CKEditorUploadingWidget())
    elif EDIT_MODE == 'code':
        content = forms.CharField(
                                  widget=CodeMirrorEditor(options={
                                                             'mode':'htmlmixed',
                                                             'width':'1170px',
                                                             'indentWithTabs':'true', 
                                                             #'indentUnit' : '4',
                                                             'lineNumbers':'true',
                                                             'autofocus':'true',
                                                             #'highlightSelectionMatches': '{showToken: /\w/, annotateScrollbar: true}',
                                                             'styleActiveLine': 'true',
                                                             'autoCloseTags': 'true',
                                                             'keyMap': CODEMIRROR_KEYMAP,
                                                             'theme':'blackboard',
                                                             #'fullScreen':'true',
                                                             },
                                                             script_template='codemirror2/codemirror_script_alapage.html',
                                                             modes=['css', 'xml', 'javascript', 'htmlmixed'],
                                                             )
                                  
                                  )
    else:
        content = forms.CharField(widget=forms.Textarea)
    content.required = False
    content.label = ""
    
    class Meta:
        model = Page
        exclude = ('edited', 'created', "editor")



# -*- coding: utf-8 -*-

from django.conf.urls import url
from alapage.views import HomepageView, PageView, PageWizardView, PagesmapView, AddPagePostView
from alapage.conf import STAFFPAGES
if STAFFPAGES is True:
    from staffpages.views import StaffpagesMapView


urlpatterns = []

if STAFFPAGES is True:
    urlpatterns.append(url(r'^staff/pages/$', StaffpagesMapView.as_view(), name="staffpages-map"))
urlpatterns.append(url(r'^sitemap/$', PagesmapView.as_view(), name="alapage-map"))
urlpatterns.append(url(r'^alapage/wizard/$', PageWizardView.as_view(), name="alapage-wizard"))
urlpatterns.append(url(r'^alapage/wizard/post/$', AddPagePostView.as_view(), name="alapage-wizard-post"))
urlpatterns.append(url(r'^(?P<url>.*?)$', PageView.as_view(), name="page-view"))
urlpatterns.append(url(r'^', HomepageView.as_view(), name="home-view"))


from __future__ import print_function
from django.core.management.base import BaseCommand, CommandError
from alapage.models import Page

class Command(BaseCommand):
    help = 'Creates a page'

    def add_arguments(self, parser):
        parser.add_argument('name', nargs='+', type=str)
        parser.add_argument('url', nargs='+', type=str)
    
    def handle(self, *args, **options):
        name = options['name'][0]
        url = options['url'][0]
        exists = Page.objects.filter(url=url).exists()
        #~ create page
        if not exists:
            Page.objects.create(url=url, title=name)
            print("Page "+name+" created")
        else:
            print("The page already exists at "+url)


# -*- coding: utf-8 -*-

from django.core.urlresolvers import reverse
from django.contrib import admin
from django.utils.translation import ugettext_lazy as _
from django.http import HttpResponseForbidden, HttpResponseRedirect
from mptt.admin import MPTTModelAdmin
from alapage.models import Page
from alapage.forms import PageAdminForm
from alapage.utils import can_see_page
from alapage.conf import USE_REVERSION
if USE_REVERSION:
    from reversion.admin import VersionAdmin


admin_class=admin.ModelAdmin
if USE_REVERSION:
    admin_class=VersionAdmin
@admin.register(Page)
class PageAdmin(MPTTModelAdmin, admin_class):
    change_list_template = "admin/alapage/page/change_list.html"
    form = PageAdminForm
    date_hierarchy = 'edited'
    search_fields = ['title','url','editor__username']
    list_display = ['url','title','published','registration_required', 'staff_only','edited','editor']
    list_select_related = ['editor']
    list_display_links = ['title','url']
    list_filter = ['created','edited','published','registration_required']
    filter_horizontal = ['users_only']
    list_select_related = ['editor']
    mptt_level_indent = 25
    #save_on_top = True

    
    def get_fieldsets(self, request, obj=None):
        super(PageAdmin, self).get_fieldsets(request, obj)
        base_fields = (('url', 'parent'),'title', 'template_name','published')
        fieldsets = (
            (None, {
                'fields': ('content',)
            }),
            (None, {
                'fields': base_fields,
            }),
            (_(u'SEO'), {
                'classes': ('collapse',),
                'fields': ('seo_keywords','seo_description')
            }),
            (_(u'Permissions'), {
                'classes': ('collapse',),
                'fields': ('registration_required','is_reserved_to_users', 'users_only', 'is_reserved_to_groups', 'groups_only', 'staff_only', 'superuser_only')
            }),
        )
        return fieldsets
    
    
    def get_readonly_fields(self, request, obj=None):
        readonly_fields = ()
        if request.user.has_perm('can_change_page_permissions') is False:
            readonly_fields = ('registration_required', 'superuser_only', 'staff_only', 'groups_only', 'users_only', 'is_reserved_to_groups', 'is_reserved_to_users')
        return readonly_fields

    def save_model(self, request, obj, form, change):
        obj.editor = request.user
        obj.save()
        return
          
    def change_view(self, request, object_id):
        page = Page.objects.filter(pk=object_id).prefetch_related('groups_only', 'users_only')[0]
        # check if the view is restricted so that only the allowed users can edit it
        user_can_see_page = can_see_page(page, request.user)
        if user_can_see_page is False:
            return HttpResponseForbidden()
        return super(PageAdmin, self).change_view(request, object_id)
    
    def response_change(self, request, obj):
        # for inline editing
        if '_inline_' in request.POST:
            return HttpResponseRedirect(reverse('page-view', kwargs={"url":obj.url}))
        else:
            return super(PageAdmin, self).response_change(request, obj)



# -*- coding: utf-8 -*-

def can_see_page(page, user):
    """
    To avoid queries pass a prefetched related page object: prefetch 'users_only' and/or 'groups_only'
    """
    if user.is_superuser:
        return True
    if page.superuser_only is True:
        return False
    if page.staff_only is True:
        if user.is_staff:
            return True
        else:
            return False
    if page.is_reserved_to_groups is True:
        user_is_allowed = False
        for group in user.groups.all():
            if group in page.groups_only.all():
                user_is_allowed = True
                break
        return user_is_allowed
    if page.is_reserved_to_users is True:
        if user in page.users_only.all():
            return True
        else:
            return False
    if page.registration_required:
        if user.is_anonymous():
            return False
    return True


# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2017-06-14 12:51
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import mptt.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('auth', '0008_alter_user_username_max_length'),
    ]

    operations = [
        migrations.CreateModel(
            name='Page',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('seo_description', models.CharField(blank=True, help_text='Short description of the page content', max_length=256, null=True, verbose_name='SEO: description')),
                ('seo_keywords', models.CharField(blank=True, help_text='List of keywords separated by commas', max_length=120, null=True, verbose_name='SEO: keywords')),
                ('url', models.CharField(db_index=True, max_length=180, verbose_name='Url')),
                ('title', models.CharField(max_length=200, verbose_name='Title')),
                ('content', models.TextField(blank=True, verbose_name='Content')),
                ('template_name', models.CharField(blank=True, help_text='If no template name is provided "alapage/default.html" will be used.', max_length=120, verbose_name='Template name')),
                ('registration_required', models.BooleanField(default=False, help_text='If this is checked, only logged-in users will be able to view the page.', verbose_name='Registration required')),
                ('edited', models.DateTimeField(auto_now=True, null=True, verbose_name='Edited')),
                ('created', models.DateTimeField(auto_now_add=True, null=True, verbose_name='Created')),
                ('published', models.BooleanField(default=True, verbose_name='Published')),
                ('staff_only', models.BooleanField(default=False, verbose_name='Staff only')),
                ('superuser_only', models.BooleanField(default=False, verbose_name='Superuser only')),
                ('is_reserved_to_groups', models.BooleanField(default=False, verbose_name='Reserved to groups')),
                ('is_reserved_to_users', models.BooleanField(default=False, verbose_name='Reserved to users')),
                ('lft', models.PositiveIntegerField(db_index=True, editable=False)),
                ('rght', models.PositiveIntegerField(db_index=True, editable=False)),
                ('tree_id', models.PositiveIntegerField(db_index=True, editable=False)),
                ('level', models.PositiveIntegerField(db_index=True, editable=False)),
                ('editor', models.ForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to=settings.AUTH_USER_MODEL, verbose_name='Edited by')),
                ('groups_only', models.ManyToManyField(blank=True, to='auth.Group', verbose_name='Reserved to some groups')),
                ('parent', mptt.fields.TreeForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='children', to='alapage.Page', verbose_name='Parent page')),
                ('users_only', models.ManyToManyField(blank=True, to=settings.AUTH_USER_MODEL, verbose_name='Reserved to some users')),
            ],
            options={
                'verbose_name': 'Page',
                'ordering': ['url'],
                'permissions': (('can_change_page_permissions', 'Can change page permissions'),),
                'verbose_name_plural': 'Page',
            },
        ),
    ]



# -*- coding: utf-8 -*-

import os
from django.core.management.base import BaseCommand, CommandError
from alapage.models import Page
from alapage.management.conf import bcolors

class Command(BaseCommand):
    help = 'Populates a page with content from direct input of file: option "-f filename.html""'

    def add_arguments(self, parser):
        parser.add_argument('content', nargs='+', type=str)
        parser.add_argument('-f',
                            action='store_true',
                            dest='file',
                            default='',
                            help='File to populate from',
                            )
    
    def handle(self, *args, **options):
        msg = 'What is the url of the page you want to populate?\n> '
        url = raw_input(msg)
        exists = Page.objects.filter(url=url).exists()
        overwrite = False
        #~ control page
        if not exists:
            print bcolors.WARNING +'!'+bcolors.ENDC+" Page "+url+" does not exist"
            msg = "Do you want the page "+url+" to be created? [Y/n]\n> "
            resp = raw_input(msg)
            if resp =='' or resp=='Y':
                msg2 = "Please type a title for the new page\n> "
                title = raw_input(msg2)
                page = Page.objects.create(url=url, title=title)
            else:
                print bcolors.FAIL +'X'+bcolors.ENDC+" Operation cancelled"
                return
        else:
            page = Page.objects.get(url=url) 
        if options['file']:
            filepath = options['content'][0]
            try:
                #~ check if the file exists
                if not os.path.isfile(filepath):
                    print bcolors.FAIL +'X'+bcolors.ENDC+" File does not exist: operation cancelled"
                    #print bcolors.FAIL +'X'+bcolors.ENDC+" Operation cancelled"
                    return
                else:
                    #~ write the file
                    with open(filepath, 'r') as content_file:
                        content = content_file.read()
            except Exception, e:
                print str(e)
        elif options['content']:
            content = options['content'][0]
        else:
            print bcolors.FAIL +'X'+bcolors.ENDC+" You must provide arguments"
        if len(page.content) > 0:
            msg = 'The page already has content: do you really want to overwrite it? [y/N] \n> '
            resp = raw_input(msg)
            if resp == 'y':
                overwrite = True
            else:
                print bcolors.FAIL +'X'+bcolors.ENDC+" Operation cancelled"
                return
        page.content = content
        page.save()
        if not overwrite:
            print bcolors.OKGREEN+'Ok'+bcolors.ENDC+" Page "+url+" has been populated"
        else:
            print bcolors.OKGREEN+'Ok'+bcolors.ENDC+" Page "+url+" has been overwrited"
        return









# -*- coding: utf-8 -*-

from django.utils.translation import ugettext_lazy as _
from django.conf import settings
from django.contrib.auth.models import User


EDIT_MODES = (
              'visual',
              'code',
              )

USER_MODEL = getattr(settings, 'AUTH_USER_MODEL', User)

USE_REVERSION=getattr(settings, 'ALAPAGE_USE_REVERSION', "reversion" in settings.INSTALLED_APPS)

BASE_TEMPLATE_PATH = getattr(settings, 'ALAPAGE_BASE_TEMPLATE_PATH', 'base.html')

EDIT_MODE = getattr(settings, 'ALAPAGE_EDIT_MODE', EDIT_MODES[0])
CODEMIRROR_KEYMAP = getattr(settings, 'ALAPAGE_CODEMIRROR_KEYMAP', 'default')

STAFFPAGES = "staffpages" in getattr(settings, 'INSTALLED_APPS')



from setuptools import setup, find_packages


version = __import__('alapage').__version__

setup(
  name = 'django-alapage',
  packages=find_packages(),
  include_package_data=True,
  version = version,
  description = 'Page management app for Django',
  author = 'synw',
  author_email = 'synwe@yahoo.com',
  url = 'https://github.com/synw/django-alapage', 
  download_url = 'https://github.com/synw/django-alapage/releases/tag/'+version,
  keywords = ['django', 'page management', 'jssor', 'slideshows'], 
  classifiers = [
        'Development Status :: 3 - Alpha',
        'Framework :: Django :: 1.9',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2.7',
    ],
  install_requires=[
        'Pillow',
        'pytz',
        'django-ckeditor',
        'django-codemirror2',
        "djangoajax",
        "django-mptt",
        "django-mptt-graph",
    ],
  zip_safe=False
)
